<?php
/**
 * @file
 * Contains Drupal\maestro_taskconsole\Controller\MaestroTaskConsoleController.
 */

namespace Drupal\maestro_taskconsole\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\maestro\Engine\MaestroEngine;
use Drupal\maestro\Utility\TaskHandler;
use Drupal\Core\Url;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Component\Serialization\Json;
use Drupal\maestro\Controller\MaestroOrchestrator;

class MaestroTaskConsoleController extends ControllerBase {
  
  /**
   * getTasks method
   * This method is called by the menu router for /taskconsole
   * The output of this method is the current user's task console.
   */
  public function getTasks($highlightQueueID = 0) {
    global $base_url;

    $config = \Drupal::config('maestro.settings');
    //before we do anything, let's see if we should be running the orchestrator through task console refreshes:
    if($config->get('maestro_orchestrator_task_console')) {
      $orchestrator = new MaestroOrchestrator();
      $orchestrator->orchestrate($config->get('maestro_orchestrator_token'));
    }
    $engine = new MaestroEngine();
    
    $form = array();
    $form['task_console_table'] = array (
      '#type' => 'table',
      '#header' => array(t('Task'), t('Flow'), t('Assigned'), t('Actions')),   
      '#empty' => t('You have no tasks.'),
      '#attributes' => array(
        'class' => ['taskconsole-tasks'],
      ),
    );

    //fetch the user's queue items
    $queueIDs = MaestroEngine::getAssignedTaskQueueIds(\Drupal::currentUser()->id());
    
    foreach($queueIDs as $queueID) {
      $highlight = '';
      $url_from_route = FALSE;
      if($highlightQueueID == $queueID){
        //set the highlight for the queue entry
        $highlight = 'maestro-highlight-task';
      }

      /*
       *  Reset the internal static cache for this queue record and then reload it
       *  Doing this because we found in certain cases it was not reflecting actual queue record
       */
      \Drupal::entityTypeManager()->getStorage('maestro_queue')->resetCache(array($queueID));
      $queueRecord = \Drupal::entityTypeManager()->getStorage('maestro_queue')->load($queueID);

      $processID = MaestroEngine::getProcessIdFromQueueId($queueID);
      $processRecord = MaestroEngine::getProcessEntryById($processID);
      
      $form['task_console_table'][$queueID]['#attributes'] = array('class' => $highlight);
      
      $form['task_console_table'][$queueID]['task'] = array(
        '#plain_text' => $queueRecord->task_label->getString(),
      );
      
      $form['task_console_table'][$queueID]['flow'] = array(
        '#plain_text' => $processRecord->process_name->getString(),
      );
      
      $form['task_console_table'][$queueID]['assigned'] = array(
        '#plain_text' =>  \Drupal::service('date.formatter')->format($queueRecord->created->getString(), 'custom', 'Y-m-d H:i:s'),
      );

      
      /*
       * If this is an interactive Maestro task, it means we show an Operations Dropbutton form element
       * This is a  button with one or more links where the links can be to a node add/edit or
       * to open up a modal window for an interactive task like a form approval action.
       *
       * We need to determine if we have any special handling for this interactive task. It could be
       * a link to an external system.
       */

      /*
       * Test to see if this is a URL that can be deduced from a Drupal route or not.
       * if it's not a route, then $url_from_route will be FALSE
       */

      $handler = $queueRecord->handler->getString();
      if($handler and !empty($handler)) {

        $handler = str_replace($base_url, '', $handler);
        $handler_type = TaskHandler::getType($handler);

        $query_options = array('queueid' => $queueID);
        $handler_url_parts = UrlHelper::parse($handler);
        $query_options += $handler_url_parts['query'];

        $templateMachineName = $engine->getTemplateIdFromProcessId($queueRecord->process_id->getString());
        $taskTemplate = $engine->getTemplateTaskByID($templateMachineName, $queueRecord->task_id->getString());
        $link = 'Execute';  // Default link title
        //Let's call a hook here to let people change the name of the link to execute the task if they so choose to do so
        \Drupal::moduleHandler()->invokeAll('maestro_task_console_interactive_link_alter', array(&$link, $taskTemplate, $queueRecord, $templateMachineName));

        $use_modal = FALSE;
        if (array_key_exists('data', $taskTemplate) && array_key_exists('modal', $taskTemplate['data']) && $taskTemplate['data']['modal'] == 'modal') {
          $use_modal = TRUE;
        }

        $links = array();

        switch($handler_type) {
          case 'external' :
            $form['task_console_table'][$queueID]['execute']['maestro_link'] =
              array(
                '#type' => 'link',
                '#title' => $this->t($link),
                '#url' => Url::fromUri($handler, array('query' => $query_options)),
              );
            break;

          case 'internal':
            $form['task_console_table'][$queueID]['execute'] = array(
              'data' => array(
                '#type' => 'operations',
                '#links' => array(
                  'maestro_link' => array(
                    'title' => $this->t($link),
                    'url' => Url::fromUserInput($handler, array('query' => $query_options)),
                  )
                )
              ),
            );

            //Let's call a hook here to let people change the name of the link to execute the task if they so choose to do so
            \Drupal::moduleHandler()->invokeAll('maestro_task_console_interactive_link_alter', array(&$link, $taskTemplate, $queueRecord, $templateMachineName));
            break;

          case 'function':
            if ($use_modal) {
              $query_options += ['modal' => 'modal'];
              $links[$link] = array(
                'title' => $this->t($link),
                'url' => Url::fromRoute('maestro.execute', $query_options),
                'attributes' => [
                  'class' => ['use-ajax'],
                  'data-dialog-type' => 'modal',
                  'data-dialog-options' => Json::encode([
                    'width' => 700,
                  ]),
                ],
              );
            }
            else {
              $query_options += ['modal' => 'notmodal'];
              $links[$link] = array(
                'title' => $this->t($link),
                'url' => Url::fromRoute('maestro.execute', $query_options),
              );
            }

            $form['task_console_table'][$queueID]['execute'] = array(
              'data' => array(
                '#type' => 'operations',
                '#links' => $links,
              ),
            );

            //Let's call a hook here to let people change the name of the link to execute the task if they so choose to do so
            \Drupal::moduleHandler()->invokeAll('maestro_task_console_interactive_link_alter', array(&$link, $taskTemplate, $queueRecord, $templateMachineName));

            break;

          default:
            $form['task_console_table'][$queueID]['execute'] = array(
              '#plain_text' => $this->t('Invalid Link'),
            );
        }

      }

      /*
       * Provide your own execution links here if you wish
       */
      \Drupal::moduleHandler()->invokeAll('maestro_task_console_alter_execution_link', array(&$form['task_console_table'][$queueID]['execute'], $taskTemplate, $queueRecord, $templateMachineName));
      
    }
    $form['#attached']['library'][] = 'maestro_taskconsole/maestro_taskconsole_css';
    return $form;
  }
  
  
}